from django import forms

from select2rocks.widgets import AjaxSelect2Widget


def label_from_instance_with_pk(obj, val):
    """Associates a label with the pk of the object that generated it.

    This is needed because we cannot rely on the order elements will appear
    in select2. By doing a binding like this, we are sure to associate a pk
    with the correct label (labels are shown, pks are submitted)."""
    return u"{pk}:{val}".format(pk=obj.pk, val=val)


class Select2ModelChoiceField(forms.ModelChoiceField):
    widget = AjaxSelect2Widget

    def __init__(self, queryset, widget, label_from_instance=None, *args, **kwargs):
        kwargs['queryset'] = queryset
        kwargs['widget'] = widget
        super(Select2ModelChoiceField, self).__init__(*args, **kwargs)
        self._label_from_instance = label_from_instance or super(Select2ModelChoiceField, self).label_from_instance
        self.widget.field = self

    def label_from_instance(self, obj):
        label = self._label_from_instance(obj)
        return label_from_instance_with_pk(obj, label)


class Select2ModelMultipleChoiceField(forms.ModelMultipleChoiceField):
    widget = AjaxSelect2Widget

    def __init__(self, queryset, widget, label_from_instance=None, *args, **kwargs):
        kwargs['queryset'] = queryset
        kwargs['widget'] = widget
        super(Select2ModelMultipleChoiceField, self).__init__(*args, **kwargs)
        if not self.widget.select2_options:
            self.widget.select2_options = {}
        self.widget.select2_options.update({'multiple': True})
        self._label_from_instance = label_from_instance or super(Select2ModelMultipleChoiceField, self).label_from_instance
        self.widget.field = self

    def label_from_instance(self, objects):
        return ','.join([label_from_instance_with_pk(obj, self._label_from_instance(obj)) for obj in objects])

    def to_python(self, values):
        return super(Select2ModelMultipleChoiceField, self).to_python(values.split(','))

    def clean(self, value):
        return super(Select2ModelMultipleChoiceField, self).clean(value.split(',') if value else None)



# -*- coding: utf-8 -*-

import os
from setuptools import setup

__version__ = '0.7.0'

README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-select2-rocks',
    version=__version__,
    packages=['select2rocks'],
    include_package_data=True,
    license='BSD License',
    description='A Django application to provide AJAX autocomplete with Select2.',
    long_description=README,
    author='Stéphane Raimbault',
    author_email='stephane.raimbault@polyconseil.fr',
    zip_safe=False,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
